from cStringIO import StringIO
import os
import PIL
from reportlab.pdfgen.canvas import Canvas
from reportlab.lib.styles import StyleSheet1, ParagraphStyle
from reportlab.lib.units import cm
from reportlab.lib import colors
from reportlab.lib import enums
from reportlab.lib.utils import ImageReader
from reportlab.platypus import Paragraph, Spacer, Frame, Table, TableStyle, PageBreak, Flowable, Image
from reportlab.platypus.doctemplate import BaseDocTemplate, NextPageTemplate, PageTemplate, ActionFlowable, FrameBreak, CurrentFrameFlowable

class MyImageReader(ImageReader):
    def __init__(self, image, filename):
        self.fileName = filename
        self._image = image
        self._width = None
        self._height = None
        self._transparent = None
        self._data = None

            
class ZopeImage(Flowable):
    """Flowables provide wrap and draw methods, however they can...."""
    def __init__(self, data, width=None, height=None, kind='direct', mask="auto"):
        """If size to draw at not specified, get it from the image."""
        self.hAlign = 'CENTER'
        self._mask = mask
        self.drawWidth = width
        self.drawHeight = height
        self._img = data

    def wrap(self, availWidth, availHeight):
        #the caller may decide it does not fit.
        return (self.drawWidth, self.drawHeight)

    def draw(self):
        self.canv.drawImage(    self._img,
                                0,
                                0,
                                self.drawWidth,
                                self.drawHeight
##                                mask=self._mask,
                                )

class MyStyleSheet(StyleSheet1):
    def __init__(self):
        StyleSheet1.__init__(self)

        self.add(ParagraphStyle(name='Normal',
                                  fontName='Helvetica',
                                  textColor= colors.black,
                                  fontSize=10,
                                  leftIndent=10,
                                  leading=12)
                   )

        self.add(ParagraphStyle(name='Heading1',
                              parent=self['Normal'],
                              fontSize=18,
                              textColor= colors.darkblue,
                              leading=22,
                              spaceAfter=6),
               alias='h1')

        self.add(ParagraphStyle(name='Footer',
                              alignment=enums.TA_RIGHT,
                              parent=self['Normal'],
                              fontSize=10),
               alias='ft')


class MyTemplate(BaseDocTemplate):
    def __init__(self, filename, templates):
        self.templates = templates
        BaseDocTemplate.__init__(self, filename)
        
    def handle_pageBegin(self):
        self._handle_pageBegin()
        template = self.templates[0]
        del self.templates[0]
        print 'switching to ' + template
        self._handle_nextPageTemplate(template)

def getpagetemplates():
    reply = []

    fh   = Frame(cm, 25*cm, 18*cm, 3*cm,  showBoundary=0)
    fpic = Frame(cm, 10*cm, 18*cm, 14*cm, showBoundary=0)    
    ftxt = Frame(cm, 3*cm, 18*cm, 6*cm, showBoundary=0)    
    fftr = Frame(0.5*cm, cm, 18*cm, 2*cm, showBoundary=0)
    
    reply.append( PageTemplate( id='Main', frames=[fh, fpic, ftxt, fftr] ) )
    return reply

def getflowables(image, name, text):
    styles = MyStyleSheet()
    [width,height] = image.size
    reply = []
    reply.append( Image( "c:\\scripts\\pdf\\logo.gif", width=17*cm, height=2.5*cm ) )
    reply.append( FrameBreak() )

    imdata = MyImageReader( image, name )
    if width > height:
        im = ZopeImage(imdata,width=17*cm, height=12.75*cm)
    else:
        im = ZopeImage(imdata,width=9.5*cm, height=12.75*cm)
    reply.append( im )
    reply.append( FrameBreak() )
    reply.append( Paragraph( name, styles['h1']) )
    reply.append( Paragraph( text, styles['Normal']) )
    reply.append( FrameBreak() )
    return reply


def makePDF(self):
    styles = MyStyleSheet()
    page = 1
    sean = '<para alignment=CENTER>Sean Crawford Sculpture</para>'

    flow = []
    doc = MyTemplate("c:\\aardvark\\clients\\galleries\\basedoc.pdf", ['Main','Main','Main','Main','Main','Main','Main','Main','Main','Main','Main','Main','Main','Main','Main','Main','Main','Main','Main','Main','Main','Main','Main','Main','Main'])
    doc.addPageTemplates( getpagetemplates() )


    for x in self.listFolderContents(contentFilter={'Type':('Photo Album','Photo',)}):
        if x.Type() == 'Photo':

#            x.checkForVariant( 'large' )
#            q = x.getPhoto('large').data
            q = x.data
            
            image = PIL.Image.open(StringIO(str(q)))
            name = x.Title()
            text = x.Description()

            for foo in getflowables( image, name, text ):
                flow.append(foo)

            flow.append( Paragraph( sean, styles['Footer']) )
            flow.append( Paragraph( "Page " + str(page), styles['Footer']) )
            flow.append( PageBreak() )
            page = page + 1

    doc.build( flow )


    
